#import <Foundation/NSData.h>
#import <Foundation/NSString.h>
#if !(TARGET_OS_IPHONE)

/**
 * This NSData category provides methods for base 64 encoding and decoding the
 * value of an NSData object and computing hashes.  Uses the OpenSSL base 64
 * encoding and decoding functionality.
 */
@interface NSData (Hash) 
/**
 * Returns a string representing the base64-encoded version of the string.
 * This operation can be reversed by sending a -base64DecodedData message to
 * the resulting string.
 */
- (NSString*) base64String;
/**
 * Returns a string containing the SHA1 hash of the data.
 */
- (NSString*) sha1;
/**
 * Returns a string containing the RIPEMD160 hash of the data.
 */
- (NSString*) ripemd160;
/**
 * Returns a string containing the MD5 hash of the data.
 */
- (NSString*) md5;
@end

@interface NSString (Base64)
/**
 * Returns an NSData object generated by assuming that the input string is
 * base64-encoded data and decoding it. 
 */
- (NSData*) base64DecodedData;
@end

#endif
